/**
 * (Re-)initializes the spreadsheet, for instance after adding rows or upgrading.
 */
function initializeSpreadsheet() {
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  ss.toast("Initializing the spreadsheet (again)...", "Please wait...", -1);
  var triggers = ScriptApp.getProjectTriggers();
  var checkForUpdateTrigger, editTrigger, refreshNecessary = false;
  // Look for an onEdit project trigger
  for (var counter = 0; counter < triggers.length; counter++) {
    if (triggers[counter].getEventType() == ScriptApp.EventType.ON_EDIT) {
      editTrigger = true;
    } else {
      if (
        (triggers[counter].getEventType() == ScriptApp.EventType.ON_OPEN) &&
        (triggers[counter].getHandlerFunction() == "checkForUpdate")
      ) {
        checkForUpdateTrigger = true;
      }
    }
  }
  // Create a new onOpen project trigger if it wasn't found previously. For some
  // weird reason or another, just putting a checkForUpdate() call in onOpen()
  // doesn't work, even after waiting for 20 seconds before calling checkForUpdate().
  if (! checkForUpdateTrigger) {
    ScriptApp.newTrigger("checkForUpdate").forSpreadsheet(ss).onOpen().create();
  }
  // Create a new onEdit project trigger if it wasn't found previously
  if (! editTrigger) {
    ScriptApp.newTrigger("whenEdited").forSpreadsheet(ss).onEdit().create();
  }
  // Make sure the locale is en_GB
  if (ss.getSpreadsheetLocale() != "en_GB") {
    ss.setSpreadsheetLocale("en_GB");
    refreshNecessary = true;
  }
  // Make sure the timezone is Europe/London
  if (ss.getSpreadsheetTimeZone() != "Europe/London") {
    ss.setSpreadsheetTimeZone("Europe/London");
    refreshNecessary = true;
  }
  // This is the main sheet
  var sites = ss.getSheetByName("Exploration sites");
  // The sheet with all the region names
  var regions = ss.getSheetByName("Regions");
  // The sheet with all the solar systems along with their regions
  var systems = ss.getSheetByName("Systems");
  var columnA = sites.getRange("A:A");
  var columnB = sites.getRange("B:B");
  var columnC = sites.getRange("C:C");
  var columnD = sites.getRange("D:D");
  var columnE = sites.getRange("E:E");
  var columnF = sites.getRange("F:F");
  var columnG = sites.getRange("G:G");
  var columnH = sites.getRange("H:H");
  var columnI = sites.getRange("I:I");
  var columnJ = sites.getRange("J:J");
  var columnK = sites.getRange("K:K");
  var columnL = sites.getRange("L:L");
  var columnM = sites.getRange("M:M");
  var columnN = sites.getRange("N:N");
  var columnO = sites.getRange("O:O");
  var columnP = sites.getRange("P:P");
  var columnQ = sites.getRange("Q:Q");
  var regionsColumnA = regions.getRange("A:A");
  var systemsColumnA = systems.getRange("A:A");
  // For some reason, row 35 has double height, while this isn't necessary, so just give it the standard height
  regions.setRowHeight(35, 17);
  // Make the Regions sheet as lean as possible
  var temp = regions.getMaxColumns();
  if (temp > 1) {
    regions.deleteColumns(2, temp - 1);
  }
  // Make the Systems sheet as lean as possible
  temp = systems.getMaxColumns();
  if (temp > 2) {
    systems.deleteColumns(3, temp - 2);
  }
  // Set the desired data validation and formatting of all columns
  var rule = SpreadsheetApp.newDataValidation().requireValueInRange(systemsColumnA).setAllowInvalid(false).build();
  columnA.setDataValidation(rule).setHorizontalAlignment("left");
  columnH.setDataValidation(rule).setHorizontalAlignment("left");
  rule = SpreadsheetApp.newDataValidation().requireValueInRange(regionsColumnA).setAllowInvalid(false).build();
  columnB.setDataValidation(rule).setHorizontalAlignment("left");
  rule = SpreadsheetApp.newDataValidation().requireDate().setAllowInvalid(false).setHelpText("Enter a valid date").build();
  columnC.setDataValidation(rule).setHorizontalAlignment("right");
  rule = SpreadsheetApp.newDataValidation().requireValueInList(["Combat", "Data", "Gas", "none", "Relic", "Unknown", "Wormhole"]).setAllowInvalid(false).build();
  columnD.setDataValidation(rule).setHorizontalAlignment("left");
  columnE.setHorizontalAlignment("left");
  rule = SpreadsheetApp.newDataValidation().requireValueInList(["yes", "no"]).setAllowInvalid(false).build();
  columnF.setDataValidation(rule).setHorizontalAlignment("left");
  columnG.setHorizontalAlignment("left");
  rule = SpreadsheetApp.newDataValidation().requireValueInList(
    [
      "A009", "A239", "A641", "A982", "B041", "B274", "B449", "B520", "B735", "C008", "C125", "C140", "C247", "C248", "C391", "C414", "D364", "D382", "D792", "D845", "E004",
      "E175", "E545", "E587", "F135", "F355", "G008", "G024", "H121", "H296", "H900", "I182", "J244", "K162", "K329", "K346", "L005", "L031", "L477", "L614", "M001", "M164",
      "M267", "M555", "M609", "N062", "N110", "N290", "N432", "N766", "N770", "N944", "N968", "O128", "O477", "O883", "P060", "Q003", "Q063", "Q317", "R051", "R259", "R474",
      "R943", "S047", "S199", "S804", "S877", "T405", "T458", "U210", "U319", "U574", "V283", "V301", "V753", "V898", "V911", "V928", "W237", "X702", "X877", "Y683", "Y790",
      "Z006", "Z060", "Z142", "Z457", "Z647", "Z971"
    ]
  ).setAllowInvalid(false).build();
  columnI.setDataValidation(rule).setHorizontalAlignment("center");
  columnJ.setHorizontalAlignment("center");
  columnK.setHorizontalAlignment("center");
  columnL.setNumberFormat("0.00").setHorizontalAlignment("center");
  rule = SpreadsheetApp.newDataValidation().requireValueInList(
    [
      "Has not yet had its stability significantly disrupted",
      "Has had its stability reduced",
      "Has had its stability critically disrupted"
    ]
  ).setAllowInvalid(false).build();
  columnM.setDataValidation(rule).setHorizontalAlignment("left");
  columnN.setHorizontalAlignment("center");
  columnO.setHorizontalAlignment("center");
  rule = SpreadsheetApp.newDataValidation().requireValueInList(
    [
      "Has not yet begun its cycle",
      "Is beginning to decay",
      "Is reaching the end",
      "Is on the verge of dissipating"
    ]
  ).setAllowInvalid(false).build();
  columnP.setDataValidation(rule).setHorizontalAlignment("left");
  columnQ.setHorizontalAlignment("center");
  sites.getRange(1, 8).setHorizontalAlignment("center");
  for (counter = 1; counter < 18; counter++) {
    sites.getRange(2, counter).setHorizontalAlignment("center");
  }
  // The Regions and Systems sheets are used by this code for data validation, but not by the user directly, so hide them
  regions.hideSheet();
  systems.hideSheet();
  var msg = "The spreadsheet is initialized (again)!";
  if (refreshNecessary) {
    msg += " Also, press F5 or reload the web page because the locale and/or timezone of this spreadsheet has been changed.";
    ss.toast("Done!", "", 1);
    Browser.msgBox("Please reload...", msg, Browser.Buttons.OK);
  } else {
    ss.toast(msg, "Done!", 5);
  }
} // function initializeSpreadsheet

/**
 * Executed by whenEdited() whenever column M changes in order to fill in the correct value in column N.
 * @param {number} row - The row where the magic has to be done
 */
function capacityInfoColumnChanged(row) {
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  var sheet = ss.getActiveSheet();
  var value = sheet.getRange(row, 13).getValue();
  var capacity = 0;
  var capacityLeft, totalCapacity;

  totalCapacity = sheet.getRange(row, 12).getValue();
  if (totalCapacity != 0) {
    totalCapacity = totalCapacity * 1000;
  }

  switch (value) {
    case "Has not yet had its stability significantly disrupted":
      if (totalCapacity != 0) {
        capacityLeft = "> " + (totalCapacity / 2);
      } else {
        capacityLeft = "> 50%";
      }
      break;
    case "Has had its stability reduced":
      if (totalCapacity != 0) {
        capacityLeft = (totalCapacity / 10) + " - " + (totalCapacity / 2);
      } else {
        capacityLeft = "10% - 50%";
      }
      break;
    case "Has had its stability critically disrupted":
      if (totalCapacity != 0) {
        capacityLeft = "< " + (totalCapacity / 10);
      } else {
        capacityLeft = "< 10%";
      }
      break;
    default:
      capacityLeft = "";
  }

  sheet.getRange(row, 14).setValue(capacityLeft).setHorizontalAlignment("center").setBackground("#d9d2e9");
} // function capacityInfoColumnChanged

/**
 * Checks if there is a newer version of the spreadsheet available and notifies the user if so.
 * @since 0.2.0
 */
function checkForUpdate() {
  var major = 0;
  var minor = 6;
  var patch = 0;
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  ss.toast("Checking for update...", "Please wait...", -1);
  var url = "https://eve-explo.appspot.com/version/";
  var result = UrlFetchApp.fetch(url);
  var output = Utilities.jsonParse(result.getContentText());
  if (output.ok) {
    ss.toast("Done!", "", 1);
    if (output.version.major != major || output.version.minor != minor || output.version.patch != patch) {
      var version = output.version.major + "." + output.version.minor + "." + output.version.patch;
      var html = HtmlService.createHtmlOutput(
        '<h1 style="font-family: Verdana, Arial, sans-serif; font-size: 14pt; font-weight: bold">Update available!</h1>' +
        '<p style="font-family: Verdana, Arial, sans-serif; font-size: 10pt">There is an update of this spreadsheet available at ' +
        '<a href="https://eve-explo.appspot.com/get/?version=latest">' +
        "https://eve-explo.appspot.com/get/?version=latest</a>. The latest version is " +
        version + ". You have version " + major + "." + minor + "." + patch + ".</p>" +
        '<button onclick="google.script.host.close()">Close</button>'
      );
      ss.show(html);
    }
  } else {
    ss.toast("Error: something went wrong fetching the latest version. Please try again later.", "Error!", 5);
  }
} // function checkForUpdate

/**
 * Executed by whenEdited() whenever column I changes in order to fill in the correct values in columns F, J, K, L and O.
 * @param {number} row - The row where the magic has to be done
 */
function idColumnChanged(row) {
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  var sheet = ss.getActiveSheet();
  var value = sheet.getRange(row, 9).getValue();
  var bookmarked = "yes";
  var target, massLimit, massCapacity, lifetime;

  switch (value) {

    /////////////
    // highsec //
    /////////////
    case "A641":
      target = "H";
      massLimit = 1000;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "B274":
      target = "H";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 24;
      break;
    case "B449":
      target = "H";
      massLimit = 1000;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "B520":
      target = "H";
      massLimit = 300;
      massCapacity = 5;
      lifetime = 24;
      break;
    case "D792":
      target = "H";
      massLimit = 1000;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "D845":
      target = "H";
      massLimit = 300;
      massCapacity = 5;
      lifetime = 24;
      break;
    case "N110":
      target = "H";
      massLimit = 20;
      massCapacity = 1;
      lifetime = 24;
      break;
    case "Q063":
      target = "H";
      massLimit = 20;
      massCapacity = 0.5;
      lifetime = 16;
      break;
    case "S047":
      target = "H";
      massLimit = 300;
      massCapacity = 3;
      lifetime = 24;
      break;

    ////////////
    // lowsec //
    ////////////
    case "A239":
      target = "L";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 24;
      break;
    case "C140":
      target = "L";
      massLimit = 1350;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "C391":
      target = "L";
      massLimit = 1800;
      massCapacity = 5;
      lifetime = 24;
      break;
    case "J244":
      target = "L";
      massLimit = 20;
      massCapacity = 1;
      lifetime = 24;
      break;
    case "N290":
      target = "L";
      massLimit = 1800;
      massCapacity = 5;
      lifetime = 24;
      break;
    case "N944":
      target = "L";
      massLimit = 1350;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "R051":
      target = "L";
      massLimit = 1000;
      massCapacity = 3;
      lifetime = 16;
      break;
    case "U210":
      target = "L";
      massLimit = 300;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "V898":
      target = "L";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;

    ////////////
    // nulsec //
    ////////////
    case "C248":
      target = "N";
      massLimit = 1800;
      massCapacity = 5;
      lifetime = 24;
      break;
    case "E545":
      target = "N";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 24;
      break;
    case "E587":
      target = "N";
      massLimit = 1000;
      massCapacity = 3;
      lifetime = 16;
      break;
    case "K329":
      target = "N";
      massLimit = 1800;
      massCapacity = 5;
      lifetime = 24;
      break;
    case "K346":
      target = "N";
      massLimit = 300;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "Q003":
      target = "N";
      massLimit = 5;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "S199":
      target = "N";
      massLimit = 1350;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "V283":
      target = "N";
      massLimit = 1000;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "Z060":
      target = "N";
      massLimit = 20;
      massCapacity = 1;
      lifetime = 24;
      break;
    case "Z142":
      target = "N";
      massLimit = 1350;
      massCapacity = 3;
      lifetime = 24;
      break;

    /////////////
    // class 1 //
    /////////////
    case "E004":
      target = "1";
      massLimit = 5;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "H121":
      target = "1";
      massLimit = 20;
      massCapacity = 0.5;
      lifetime = 16;
      break;
    case "P060":
      target = "1";
      massLimit = 20;
      massCapacity = 0.5;
      lifetime = 16;
      break;
    case "Q317":
      target = "1";
      massLimit = 20;
      massCapacity = 0.5;
      lifetime = 16;
      break;
    case "V301":
      target = "1";
      massLimit = 20;
      massCapacity = 0.5;
      lifetime = 16;
      break;
    case "Y790":
      target = "1";
      massLimit = 20;
      massCapacity = 0.5;
      lifetime = 16;
      break;
    case "Z647":
      target = "1";
      massLimit = 20;
      massCapacity = 0.5;
      lifetime = 16;
      break;
    case "Z971":
      target = "1";
      massLimit = 20;
      massCapacity = 0.1;
      lifetime = 16;
      break;

    /////////////
    // class 2 //
    /////////////
    case "C125":
      target = "2";
      massLimit = 20;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "D364":
      target = "2";
      massLimit = 300;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "D382":
      target = "2";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "G024":
      target = "2";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "I182":
      target = "2";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "L005":
      target = "2";
      massLimit = 5;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "N766":
      target = "2";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "R943":
      target = "2";
      massLimit = 300;
      massCapacity = 0.75;
      lifetime = 16;
      break;

    /////////////
    // class 3 //
    /////////////
    case "C247":
      target = "3";
      massLimit = 200;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "L477":
      target = "3";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "M267":
      target = "3";
      massLimit = 300;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "N968":
      target = "3";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "O477":
      target = "3";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "O883":
      target = "3";
      massLimit = 20;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "X702":
      target = "3";
      massLimit = 300;
      massCapacity = 1;
      lifetime = 24;
      break;
    case "Z006":
      target = "3";
      massLimit = 5;
      massCapacity = 1;
      lifetime = 16;
      break;

    /////////////
    // class 4 //
    /////////////
    case "E175":
      target = "4";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "M001":
      target = "4";
      massLimit = 5;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "M609":
      target = "4";
      massLimit = 20;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "O128":
      target = "4";
      massLimit = 300;
      massCapacity = 1;
      lifetime = 24;
      break;
    case "T405":
      target = "4";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "X877":
      target = "4";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "Y683":
      target = "4";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "Z457":
      target = "4";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;

    /////////////
    // class 5 //
    /////////////
    case "C008":
      target = "5";
      massLimit = 5;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "H296":
      target = "5";
      massLimit = 1350;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "H900":
      target = "5";
      massLimit = 300;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "L614":
      target = "5";
      massLimit = 20;
      massCapacity = 1;
      lifetime = 24;
      break;
    case "M555":
      target = "5";
      massLimit = 1000;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "N062":
      target = "5";
      massLimit = 300;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "N432":
      target = "5";
      massLimit = 1350;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "N770":
      target = "5";
      massLimit = 300;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "V911":
      target = "5";
      massLimit = 1350;
      massCapacity = 3;
      lifetime = 24;
      break;

    /////////////
    // class 6 //
    /////////////
    case "A982":
      target = "6";
      massLimit = 300;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "B041":
      target = "6";
      massLimit = 300;
      massCapacity = 5;
      lifetime = 48;
      break;
    case "G008":
      target = "6";
      massLimit = 5;
      massCapacity = 1;
      lifetime = 16;
      break;
    case "R474":
      target = "6";
      massLimit = 300;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "S804":
      target = "6";
      massLimit = 20;
      massCapacity = 1;
      lifetime = 24;
      break;
    case "U319":
      target = "6";
      massLimit = 1800;
      massCapacity = 3;
      lifetime = 48;
      break;
    case "U574":
      target = "6";
      massLimit = 300;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "V753":
      target = "6";
      massLimit = 1350;
      massCapacity = 3;
      lifetime = 24;
      break;
    case "W237":
      target = "6";
      massLimit = 1350;
      massCapacity = 3;
      lifetime = 24;
      break;

    //////////////
    // class 13 //
    //////////////
    case "A009":
      target = "13";
      massLimit = 20;
      massCapacity = 0.5;
      lifetime = 16;
      break;

    /////////////////////
    // Drifters w-space //
    /////////////////////
    case "B735":
      target = "Barbican";
      massLimit = 300;
      massCapacity = 0.75;
      lifetime = 16;
      break;
    case "C414":
      target = "Conflux";
      massLimit = 300;
      massCapacity = 0.75;
      lifetime = 16;
      break;
    case "R259":
      target = "Redoubt";
      massLimit = 300;
      massCapacity = 0.75;
      lifetime = 16;
      break;
    case "S877":
      target = "Sentinel";
      massLimit = 300;
      massCapacity = 0.75;
      lifetime = 16;
      break;
    case "V928":
      target = "Vidette";
      massLimit = 300;
      massCapacity = 0.75;
      lifetime = 16;
      break;

    ///////////////////
    // Thera w-space //
    ///////////////////
    case "F135":
      target = "Thera";
      massLimit = 300;
      massCapacity = 0.75;
      lifetime = 16;
      break;
    case "F355":
      target = "Thera";
      massLimit = 20;
      massCapacity = 0.1;
      lifetime = 16;
      break;
    case "L031":
      target = "Thera";
      massLimit = 1000;
      massCapacity = 3;
      lifetime = 16;
      break;
    case "M164":
      target = "Thera";
      massLimit = 300;
      massCapacity = 2;
      lifetime = 16;
      break;
    case "T458":
      target = "Thera";
      massLimit = 20;
      massCapacity = 0.5;
      lifetime = 16;
      break;

    default:
      target = "";
      massLimit = "";
      massCapacity = "";
      lifetime = "";
      bookmarked = "";
  }
  sheet.getRange(row, 6).setValue(bookmarked);
  sheet.getRange(row, 9).setHorizontalAlignment("center");
  sheet.getRange(row, 10).setValue(target).setHorizontalAlignment("center").setBackground("#d9d2e9");
  sheet.getRange(row, 11).setValue(massLimit).setHorizontalAlignment("center").setBackground("#d9d2e9");
  sheet.getRange(row, 12).setValue(massCapacity).setHorizontalAlignment("center").setBackground("#d9d2e9");
  if (typeof massCapacity == "number") {
    sheet.getRange(row, 12).setNumberFormat("0.00");
  } else {
    sheet.getRange(row, 12).setNumberFormat("");
  }
  sheet.getRange(row, 15).setValue(lifetime).setHorizontalAlignment("center").setBackground("#d9d2e9");
} // function idColumnChanged

/**
 * Amazingly, does exactly what it says: jump to the next empty row in the spreadsheet. Triggered by "Spreadsheet → Jump to next empty row" in the menu.
 */
function jumpToNextEmptyRow() {
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  var sheet = ss.getActiveSheet();
  var lastRow = sheet.getLastRow() + 1;
  sheet.setActiveRange(sheet.getRange(lastRow, 1));
  ss.toast(
    "Jumped to row " + lastRow +
    "... Now, don't forget to press Escape on your keyboard and you can start inputting new rows with data from scanned signatures!",
    "",
    8
  );
} // function jumpToNextEmptyRow

/**
 * Automagically triggered when opening the spreadsheet. Makes the Spreadsheet items in the menu.
 */
function onOpen() {
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  var menuEntries = [];
  menuEntries.push({name: "Jump to next empty row", functionName: "jumpToNextEmptyRow"});
  menuEntries.push({name: "(Re-)initialize", functionName: "initializeSpreadsheet"});
  menuEntries.push({name: "Check for update", functionName: "checkForUpdate"});
  ss.addMenu("Spreadsheet", menuEntries);
} // function onOpen

/**
 * Executed by whenEdited() whenever columns D, I, M or P change. Sets column C to a nice EVE Time date/time.
 * @param {number} row - The row where the date/time should be filled in/updated
 */
function setDateTimeColumn(row) {
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  var sheet = ss.getActiveSheet();
  var newDate = new Date();
  var formattedDate = Utilities.formatDate(newDate, "GMT", "yyyy/MM/dd HH:mm");
  sheet.getRange(row, 3).clearFormat().setValue(formattedDate).setNumberFormat("yyyy/MM/dd HH:mm").setBackground("#d9d2e9");
  return;
  var now = new Date();
  sheet.getRange(row, 3).setValue(
    now.getUTCDate() + "/" +
    (now.getUTCMonth() + 1) + "/" +
    now.getUTCFullYear() + " " +
    now.getUTCHours() + ":" +
    now.getUTCMinutes()
  ).setNumberFormat("dd/MM/yyyy HH:mm").setBackground("#d9d2e9");
} // function setDateTimeColumn

/**
 * Executed by whenEdited() whenever column A changes. Sets column B with the region for the solar system in column A.
 * @param {number} row - The row where the magic has to be done
 */
function solarSystemColumnChanged(row) {
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  ss.toast("Determining region...", "Just a moment", -1);
  var sheet = ss.getActiveSheet();
  var url = "https://eve-explo.appspot.com/region/";
  var system = sheet.getRange(row, 1).getValue();
  var payload = {
    "system": system
  };
  var options = {
    "method": "post",
    "payload": payload
  }
  var result = UrlFetchApp.fetch(url, options);
  var output = Utilities.jsonParse(result.getContentText());
  if (output.ok) {
    sheet.getRange(row, 2).setValue(output.region);
    ss.toast("Done!", "", 3);
  } else {
    ss.toast("Error: " + output.result, "Error!", 5);
  }
} // function solarSystemColumnChanged

/**
 * Executed by whenEdited() whenever column P changes. Sets column Q with the approximate time left for this wormhole.
 * @param {number} row - The row where the magic has to be done
 */
function timeInfoColumnChanged(row) {
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  var sheet = ss.getActiveSheet();
  var value = sheet.getRange(row, 16).getValue();
  var timeLeft;

  switch (value) {
    case "Has not yet begun its cycle":
      timeLeft = "> 24";
      break;
    case "Is beginning to decay":
      timeLeft = "< 24";
      break;
    case "Is reaching the end":
      timeLeft = "< 4";
      break;
    case "Is on the verge of dissipating":
      timeLeft = "≈ 0 !";
      break;
    default:
      timeLeft = "";
  }
  sheet.getRange(row, 17).setValue(timeLeft).setHorizontalAlignment("center").setBackground("#d9d2e9");
} // function timeInfoColumnChanged

/**
 * Automatically triggered when the spreadsheet changes (is edited). Determines which row/column has been changed and executes the corresponding functions.
 * @param {Object} e - Event parameter with information about the event
 */
function whenEdited(e) {
  var ss = SpreadsheetApp.getActiveSpreadsheet();
  var sheet = ss.getActiveSheet();
  var row = e.range.getRow();

  switch (e.range.getColumn()) {
    case 1:
      if (sheet.getName() == "Exploration sites") {
        if (sheet.getRange(row, 1).getValue() == "") {
          sheet.getRange(row, 2).setValue("");
        } else {
          solarSystemColumnChanged(row);
        }
      }
      break;
    case 4:
      if (sheet.getRange(row, 4).getValue() == "") {
        sheet.getRange(row, 3).setValue("");
      } else {
        setDateTimeColumn(row);
      }
      break;
    case 5:
      sheet.getRange(row, 5).setValue(sheet.getRange(row, 5).getValue().toUpperCase());
      break;
    case 6:
      if (sheet.getRange(row, 6).getValue() == "yes") {
        setDateTimeColumn(row);
      }
      break;
    case 9:
      if (sheet.getRange(row, 9).getValue() == "") {
        sheet.getRange(row, 3).setValue("");
        sheet.getRange(row, 4).setValue("");
        sheet.getRange(row, 6).setValue("");
        sheet.getRange(row, 13).setValue("");
        sheet.getRange(row, 14).setValue("");
        sheet.getRange(row, 15).setValue("");
        sheet.getRange(row, 16).setValue("");
        sheet.getRange(row, 17).setValue("");
      } else {
        if (sheet.getRange(row, 4).getValue() != "Wormhole") {
          setDateTimeColumn(row);
          sheet.getRange(row, 4).setValue("Wormhole");
          sheet.getRange(row, 6).setValue("yes");
        }
      }
      idColumnChanged(row);
      break;
    case 13:
      capacityInfoColumnChanged(row);
      setDateTimeColumn(row);
      break;
    case 16:
      timeInfoColumnChanged(row);
      setDateTimeColumn(row);
  }
} // function whenEdited

/* vim: set filetype=javascript: */